#ifndef __HIERBOX_H
#define __HIERBOX_H 1
//************************************************************************
// 	HIERBOX.H
//************************************************************************
//
//  Purpose:	Hierarchical listbox
//
//  Author:	ABRI Datenverarbeitung GesmbH
//		Andreas Vogel
//		Miltnerweg 17/14/19
//		A-1110 Vienna, Austria
//		phone/fax/modem: 0043 1 76 23 35
//		handy:		 0043 0663 802 538
//		CIS:		 100343,2704
//
//  History:
//
//    	94-07-25  V1.00	created from MSNCD V7.0
//

#include <owl\owlpch.h>
#include <owl\listbox.h>
#include <owl\module.h>
#include <classlib\arrays.h>
#include <ctl3d.h>

#include <string.h>

#if __TURBOC__ >= 0x410
#define TResID TResId
#endif

class cHierListBox : public TListBox
{
  HDC       hdcMem;
  HBITMAP   hbmIcons;
  HBITMAP   hbmMem;
  int       nBitmapHeight;
  int       nBitmapWidth;
  int       nBitmapRows, nBitmapColumns;
  int       nTextHeight;
  int       nLineHeight;
  BOOL      bLines;            	// paint connecting lines
  int	    Ok;
  float	    XLinePos;		// defines the x-position of vertical line,
				// refered to the bitmap

  class HierListBoxData
  {
  public:
    char	Item[40];
    DWORD	Data;
    HBITMAP	hbm;

    HierListBoxData() { *Item = NULL; hbm=0; }

    HierListBoxData(const char *text, const DWORD data, const HBITMAP h=0) {
		  strncpy(Item, text, 39);
		  *(Item+39)=0;
		  Data = data;
		  hbm = h; }

    ~HierListBoxData() {}

    int operator == (const HierListBoxData &d) const {
		  return  !strcmp(d.Item, Item) &&
			  Data == d.Data; }
  };

  TArray<HierListBoxData> Content;  // duplicated contents of ListBox, cause
				    // not all items are shown in listbox

  enum { XBMPOFFSET = 2 };

public:
	cHierListBox(TWindow *Parent,
		     int      Id,
		     int      BitmapId,
		     int      rows,
		     int      columns,
		     int      loins = TRUE,
		     TModule *modul = NULL);   // take the application ones
					       // as default

      cHierListBox::cHierListBox(TWindow * Parent,
				 int       Id,
				 char *	   DefaultBitmap,
				 int	   loins = TRUE,
				 TModule * modul = ::Module);

	~cHierListBox();

  //overloaded:
  void	SetupWindow();

  //user message:
  void	EvDrawItem(UINT ctrlId, DRAWITEMSTRUCT far & draw);
  void	EvMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT far & draw);
  void	EvLButtonDown(UINT, TPoint &);
  void	EvChar(UINT, UINT, UINT);

  LRESULT	DlgSubclass(WPARAM, LPARAM);

  //user:
  int	Action();

  void	AddItem(const char *str,
		const int   level,
		const BOOL  foldup = FALSE,
		const BOOL  selectable = FALSE);

  void	AddItem(const char *BitmapFile,
		const char *str,
		const int   level,
		const BOOL  foldup = FALSE,
		const BOOL  selectable = FALSE);

  void	SetupItemData(BOOL foldup = FALSE);

  void	GetSelString(char *str, int max) {
		strncpy(str,
			Content[GetContentSelIndex()].Item,
			max); }

  int	GetContentSelIndex() {
		return GetIndex(GetItemData(GetSelIndex())); }

protected:

  DWORD RGB2BGR(DWORD rgb) {
		return RGB(GetBValue(rgb),
			   GetGValue(rgb),
			   GetRValue(rgb)); }

  DWORD RGB2BGR(TColor &c) {
		return c.Rgb(); }

  void	ShowKids(WORD, WORD);

  void  OnDrawItem(DRAWITEMSTRUCT far & draw,
		   int   	nLevel,
		   DWORD 	dwConnectLevel,
		   char  *	szText,
		   HBITMAP 	hbm,
		   int   	nRow,
		   int   	nColumn);

  void	FastRect(HDC hDC, int x, int y, int cx, int cy) {
		RECT rc;

		rc.left = x;	rc.right = x+cx;
		rc.top = y; 	rc.bottom = y+cy;

		ExtTextOut(hDC, x, y, ETO_OPAQUE, &rc, NULL, 0, NULL);  }


  // container-data functions
  //-------------------------

  void 	OpenItem(int);
  void 	CloseItem(int);

  BOOL 	IsOpened(DWORD w) {
		return (Content[GetIndex(w)].Data & 0x80000000L)!=0; }

  WORD	GetIndex(DWORD w) {
		return (WORD) (w >> 16) & 0x7FFF; }

  int	Foldable(int);
  int	LastChild(int);
  int	PrevParent(int idx);

  int	GetLevel(const int idx) {
		if (idx > Content.GetItemsInContainer())
		  return -1;
		else
		  return GetLevel(Content[idx].Data); }

  int	GetLevel(const DWORD w) {
		return (int) (w >> 8) & 0xFF; }


DECLARE_RESPONSE_TABLE(cHierListBox);
};


//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

class cHierDialog : public TDialog

//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
// Hierachical listbox dialog with ONE hier-listbox
{
public:
  cHierListBox	*ListBox;

	cHierDialog(TWindow *w, TResID id, TModule *m = NULL) :
		TDialog(w,id,m) {
			ListBox=NULL; }

  void	SetupWindow();

  void	EvMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT far & draw) {
		if (ListBox)
		  ListBox->EvMeasureItem(ctrlId, draw); }

  void	EvDrawItem(UINT ctrlId, DRAWITEMSTRUCT far & draw) {
		if (ListBox)
		  ListBox->EvDrawItem(ctrlId, draw); }

DECLARE_RESPONSE_TABLE(cHierDialog);
};

#endif


